@@ -1,6 +1,6 @@
 package LWP::Conn::HTTP; # An HTTP Connection class
 
-# $Id: HTTP.pm,v 1.23 1998/03/31 18:38:29 aas Exp $
+# $Id: HTTP.pm,v 1.26 1998/04/06 18:35:31 aas Exp $
 
 # Copyright 1997-1998 Gisle Aas.
 #
@@ -164,21 +164,40 @@ sub new_request
 		if *$self->{'lwp_req_count'} == *$self->{'lwp_req_limit'};
 	    if (@TE) {
 		push(@conn_header, "TE");
-		$req->header(TE => join(", ", @TE));
+		$req->header(TE => join(",", @TE));
 	    }
 	}
-	$req->header("Connection" => join(", ", @conn_header)) if @conn_header;
+	if (@conn_header) {
+	    $req->header("Connection" => join(",", @conn_header));
+	} else {
+	    $req->remove_header("Connection");
+	}
+
+	my $cont_ref = $req->content_ref;
+	$cont_ref = $$cont_ref if ref($$cont_ref);
+	if (ref($cont_ref) eq "CODE") {
+	    if (my $len = $req->header("Content-Length")) {
+		*$self->{'lwp_wlen'} = $len + 0;
+	    } else {
+		# must use chunked encoding for the request content
+		$req->push_header("Transfer-Encoding", "chunked");
+		*$self->{'lwp_wlen'}  = 0;
+	    }
+	    *$self->{'lwp_wdyn'} = $cont_ref;  # dynamic content
+	    $cont_ref = \"";   #"; make sure a complete header is sent first.
+	} else {
+	    my $len = length($$cont_ref);
+	    $req->header("Content-Length" => $len) if $len;
+	}
+
 	$req->scan(sub { push(@rlines, "$_[0]: $_[1]") });
-	# XXX if $req->content is a code reference, we should probably
-	# arrange for something clever to happen that involves chunked
-	# encoding.
-	push(@rlines, "", $req->content);
+	push(@rlines, "", $$cont_ref);
 	push(@{ *$self->{'lwp_req'} }, $req);
 	*$self->{'lwp_wbuf'} = join("\015\012", @rlines);
 	mainloop->writable($self);
 	return $req;
     }
-    return undef;
+    return;
 }
 
 
@@ -218,10 +237,62 @@ sub stop
 }
 
 # EventLoop callbacks
-sub writable { shift->_error("Writable connection"); }
 sub readable { shift->_error("Readable connection"); }
 sub inactive { shift->_error("Inactive connection"); }
 
+sub writable
+{
+    my $self = shift;
+    my $buf = \ *$self->{'lwp_wbuf'};
+    my $n = syswrite($self, $$buf, length($$buf));
+    if (!defined($n) || $n == 0) {
+	$self->_error("Bad write: $!");
+    } else {
+	print STDERR "WROTE $n bytes\n" if $LWP::Conn::HTTP::DEBUG;
+	if ($n < length($$buf)) {
+	    substr($$buf, 0, $n) = "";  # get rid of this
+	} else {
+	    # Check if we are generating dynamic content
+	    if (my $dyn = *$self->{'lwp_wdyn'}) {
+		my $chunk = &$dyn();
+		my $clen  = length($chunk);
+
+		if (my $len = *$self->{'lwp_wlen'}) {
+		    # we are generating content with the specified length
+		    if ($clen > $len) {
+			# chunk to large, truncate it
+			substr($chunk, $len) = '';
+			$clen = $len;
+		    } elsif ($clen == 0) {
+			$self->_error("Short dynamic request content ($len bytes missing)");
+			# Other possibilities is to fill request
+			# content with some random padding or to
+			# just continue to call the callback routine
+			# until we have enough.
+			return;
+		    }
+		    *$self->{'lwp_wbuf'} = $chunk;
+		    $len -= $clen;
+		    *$self->{'lwp_wlen'} = $len;
+		    delete *$self->{'lwp_wdyn'} unless $len;
+		} else {
+		    # we are using chunked transfer encoding for this request
+		    *$self->{'lwp_wbuf'} = join("\015\012",
+						sprintf("%x", $clen),
+						$chunk, "");
+		    delete *$self->{'lwp_wdyn'} if $clen == 0;
+		}
+		return;
+	    }
+	    # request sent
+	    delete *$self->{'lwp_wbuf'};
+	    # try to start a new one?
+	    $self->new_request or mainloop->writable($self, undef);
+	}
+    }
+}
+
+
 sub _error
 {
     my($self, $msg) = @_;
@@ -247,13 +318,14 @@ sub _error
 	    # return immediately.
 	    $cur_req->response_done($res);
 	} else {
-	    $cur_req->gen_response(590, "No response");
+	    $cur_req->gen_response(590, "No response", $msg);
 	}
     } else {
 	$mgr->connection_closed($self);
     }
 }
 
+
 sub response_data
 {
     my $self = shift;
@@ -277,6 +349,7 @@ sub response_data
 
 
 
+
 package LWP::Conn::HTTP::Connecting;
 use base qw(LWP::Conn::HTTP);
 
@@ -343,27 +416,6 @@ use LWP::MainLoop qw(mainloop);
 require HTTP::Response;
 
 
-sub writable
-{
-    my $self = shift;
-    my $buf = \ *$self->{'lwp_wbuf'};
-    my $n = syswrite($self, $$buf, length($$buf));
-    if (!defined($n) || $n == 0) {
-	$self->_error("Bad write: $!");
-    } else {
-	print STDERR "WROTE $n bytes\n" if $LWP::Conn::HTTP::DEBUG;
-	if ($n < length($$buf)) {
-	    substr($$buf, 0, $n) = "";  # get rid of this
-	} else {
-	    # request sent
-	    delete *$self->{'lwp_wbuf'};
-	    # try to start a new one?
-	    mainloop->writable($self, undef) unless $self->new_request;
-	}
-    }
-}
-
-
 sub readable
 {
     my $self = shift;
@@ -445,6 +497,19 @@ sub check_rbuf
 	return;
     }
 
+    my $req = $self->current_request;
+
+    if ($code == 101) {  # SWITCHING PROTOCOL
+	# XXX Should check for pipelining
+	mainloop->forget($self);
+	(delete *$self->{'lwp_mgr'})->connection_closed($self); # a white lie
+	bless $self, "IO::Socket::INET";  # downgrade
+	$res->{'101_socket'} = $self;     # a header would be more visible
+	$res->content($$buf);             # if we read too much
+	$req->response_done($res);
+	return;
+    }
+
     if ($code >= 100 && $code <= 199) {
 	print STDERR "Info response ($code)\n" if $LWP::Conn::HTTP::DEBUG;
 	# XXX: should we store $res anywhere or just forget about it?
@@ -453,11 +518,18 @@ sub check_rbuf
     }
 
     *$self->{'lwp_res'} = $res;
-    my $req = $self->current_request;
     $res->request($req);
     return unless $self->response_data($req, "", $res);
     #print $res->as_string if $LWP::Conn::HTTP::DEBUG;
 
+    if ($code >= 400 && $code <= 599 &&  # we got an error and
+	*$self->{'lwp_wdyn'} &&          # we are still sending dynamic content
+	@{ *$self->{'lwp_req'} } == 1    # for request with error response
+       ) {
+	# make sure it gets terminated on next opportunity.
+	*$self->{'lwp_wdyn'} = sub { "" };
+    }
+
     # Determine how to find the end of message
     if ($req->method eq "HEAD" || $code =~ /^(?:1\d\d|[23]04)$/) {
 	$self->state("ContLen");
